﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;
using Less;
using Less.Core;
using Less.Core.Loggers;
using Less.Core.Model;
using Less.SRV;
using Less.Tools;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Test
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class LessFramework
    {
        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }
        [TestMethod]
        public void Initialize()
        {
            Initializer();
        }

        public void Initializer()
        {
            try
            {
                Log.Loggers.Add(new FileLogger());
                Status.LoadStatus();
                Config.CanUseDatabase = true;
                Log.Loggers.Add(new DatabaseLogger());
            }
            catch (Exception e)
            {
                Log.Fatal(e);
            }
        }

        /// <summary>
        /// Tests the Log and Config systems
        /// </summary>
        [TestMethod]
        public void LogConfig()
        {
            try
            {
                try
                {
                    Initializer();
                    Log.Debug("Max log size in Kb:" + FileLogger.MaxLogSizeKb);
                    Log.Info("Program Started");
                    Log.Info(MailLogger.LogMailDestination);
                    Log.Info(FileLogger.MaxLogSizeKb.ToString());
                    Parallel.For(0,
                                 10,
                                 (i) =>
                                     {
                                         Log.Info("Number: " + i + " at " + DateTime.Now.Millisecond + " from threadId:" + Thread.CurrentThread.ManagedThreadId);
                                         Log.Warning("FileloggerSize: " + FileLogger.LastSize + " at " + DateTime.Now.Millisecond + " from threadId:" +
                                                     Thread.CurrentThread.ManagedThreadId);
                                     });
                    Log.Warning("Network not found. Saving data locally");
                    FailMethod();
                }
                catch (Exception e)
                {
                    Log.Error(e);
                }
            }
            catch (Exception e)
            {
                Log.Fatal(e, EntryCategories.Test, "tester");
                Assert.Fail(e.ToString());
            }
            Thread.Sleep(1000);
        }

        private static void FailMethod()
        {
            //Data.Database.Execute("DROP TABLE " + ConfigurationDatabaseManager.TableName);
            Log.Info(MailLogger.LogMailDestination); // Should raise an exception because configuration table has been dropped
        }

        [TestMethod]
        public void JSONSerialization()
        {
            var test = new Test {Message = "this is a test", Array = new string[3] {"one", "two", "three"}, List = new List<string> {"a", "b", "c"}};
            string serial = test.Serialize();
            var test2 = serial.Deserialize<Test>();
            string path = @"c:\logs\test.json";
            test2.Serialize(path);
            var test3 = Functions.Deserialize<Test>(path);
            Assert.AreEqual(test.Message, test3.Message);
            Thread.Sleep(1000);
        }

        [TestMethod]
        public void Extensions()
        {
            string longName = "LessFramework";
            string shortName = "Less";
            Assert.AreEqual(longName.Left(3), shortName.Left(3));
            // shortName.Left(6) does not throw an IndexOutOfRange Exception !!!
            Assert.AreNotEqual(longName.Left(6), shortName.Left(6));
            Thread.Sleep(1000);
        }

        [TestMethod]
        public void DataBaseLogging()
        {
            var dataBaseLogger = new DatabaseLogger();
            Log.Loggers.Add(dataBaseLogger);
            Log.Debug("Test debug");
            Log.Info("Info Test");
            Log.Warning("Warning Test");
            Log.Error(new Exception("This is an exception"));
            Log.Fatal(new Exception("This is THE Exception"));
            var log = new LogSRV("lessConnection");
            List<LogEntry> logEntries = log.GetLast(10);
            foreach (LogEntry logEntry in logEntries)
            {
                Log.Debug(logEntry.Serialize().Replace("\"", "'"));
            }
            Thread.Sleep(1000);
        }

        [TestMethod]
        public void UsersRoles()
        {
            try
            {
                var securitySRV = new SecuritySRV();
                try
                {
                    // Create Roles
                    securitySRV.InsertRoleByName("God");
                    securitySRV.InsertRoleByName("Administrator");
                    securitySRV.InsertRoleByName("User");
                    securitySRV.InsertRoleByName("Guest");
                    // Create Users
                    securitySRV.InsertUser(new User
                                               {
                                                   UserName = "JackShephard",
                                                   Email = "JackShephard@island.com",
                                                   LastActivityDate = DateTime.Now,
                                                   LastLoginDate = DateTime.Now,
                                                   CreationDate = DateTime.Now,
                                                   LastLockoutDate = DateTime.Now,
                                                   LastPasswordChangedDate = DateTime.Now,
                                                   Password = "losties"
                                               });
                    securitySRV.InsertUser(new User
                                               {
                                                   UserName = "KateAusten",
                                                   Email = "KateAusten@island.com",
                                                   LastActivityDate = DateTime.Now,
                                                   LastLoginDate = DateTime.Now,
                                                   CreationDate = DateTime.Now,
                                                   LastLockoutDate = DateTime.Now,
                                                   LastPasswordChangedDate = DateTime.Now,
                                                   Password = "losties"
                                               });
                    securitySRV.InsertUser(new User
                                               {
                                                   UserName = "BenjaminLinus",
                                                   Email = "BenjaminLinus@dharma.com",
                                                   LastActivityDate = DateTime.Now,
                                                   LastLoginDate = DateTime.Now,
                                                   CreationDate = DateTime.Now,
                                                   LastLockoutDate = DateTime.Now,
                                                   LastPasswordChangedDate = DateTime.Now,
                                                   Password = "losties"
                                               });
                    securitySRV.InsertUser(new User
                                               {
                                                   UserName = "JulietBurke",
                                                   Email = "JulietBurke@dharma.com",
                                                   LastActivityDate = DateTime.Now,
                                                   LastLoginDate = DateTime.Now,
                                                   CreationDate = DateTime.Now,
                                                   LastLockoutDate = DateTime.Now,
                                                   LastPasswordChangedDate = DateTime.Now,
                                                   Password = "losties"
                                               });
                    securitySRV.InsertUser(new User
                                               {
                                                   UserName = "HugoReyes",
                                                   Email = "HugoReyes@island.com",
                                                   LastActivityDate = DateTime.Now,
                                                   LastLoginDate = DateTime.Now,
                                                   CreationDate = DateTime.Now,
                                                   LastLockoutDate = DateTime.Now,
                                                   LastPasswordChangedDate = DateTime.Now,
                                                   Password = "losties"
                                               });
                    ;
                    securitySRV.InsertUser(new User
                                               {
                                                   UserName = "JohnLocke",
                                                   Email = "JohnLocke@others.com",
                                                   LastActivityDate = DateTime.Now,
                                                   LastLoginDate = DateTime.Now,
                                                   CreationDate = DateTime.Now,
                                                   LastLockoutDate = DateTime.Now,
                                                   LastPasswordChangedDate = DateTime.Now,
                                                   Password = "losties"
                                               });
                    securitySRV.InsertUser(new User
                                               {
                                                   UserName = "Sawyer",
                                                   Email = "Sawyer@island.com",
                                                   LastActivityDate = DateTime.Now,
                                                   LastLoginDate = DateTime.Now,
                                                   CreationDate = DateTime.Now,
                                                   LastLockoutDate = DateTime.Now,
                                                   LastPasswordChangedDate = DateTime.Now,
                                                   Password = "losties",
                                                   FailedPasswordAttemptWindowStart = DateTime.Now,
                                                   FailedPasswordAnswerAttemptWindowStart = DateTime.Now
                                               });
                    // Add Users To Roles
                    securitySRV.AddUsersToRoles(new[] {"JohnLocke", "BenjaminLinus", "JackShephard", "KateAusten", "JulietBurke"}, new[] {"User"});
                    securitySRV.AddUsersToRoles(new[] {"JohnLocke", "BenjaminLinus"}, new[] {"Administrator"});
                    securitySRV.AddUsersToRoles(new[] {"HugoReyes"}, new[] {"Guest"});
                }
                catch (RecordAlreadyExistsException e)
                {
                    Log.Warning(e.ToLogString());
                }
                catch (Exception e)
                {
                    Log.Error(e);
                }
                // List Roles of an User
                List<Role> roles = securitySRV.SelectRolesByUserName("JohnLocke");
                if (roles != null)
                {
                    foreach (Role roleRecord in roles)
                    {
                        Trace.WriteLine(roleRecord.Name);
                    }
                }
                // List Users of a Role
                List<User> users = securitySRV.SelectUsersByRoleName("User");
                if (users != null)
                {
                    foreach (User userRecord in users)
                    {
                        Trace.TraceInformation(userRecord.UserName + " : " + userRecord.Identifier);
                    }
                }
                // IsUserInRole
                Trace.TraceInformation(string.Format("Is {0} in Rol {1} ? {2}", "JackShephard", "User", securitySRV.IsUserInRole("JackShephard", "User")));
                Trace.WriteLine(string.Format("Is {0} in Rol {1} ? {2}", "JackShephard", "User", securitySRV.IsUserInRole("HugoReyes", "Administrator")));
            }
            catch (Exception e)
            {
                Log.Error(e, EntryCategories.Test, "tester");
                Assert.Fail(e.ToLogString());
            }
            Thread.Sleep(1000);
        }

        [TestMethod]
        public void Metrics()
        {
            MetricsSingle();
            MetricsParallel();
            Thread.Sleep(10000);
        }

        public void MetricsSingle()
        {
            long pId = Log.ProcessStart(processName: "SingleThread");
            DateTime start = DateTime.Now;
            for (int i = 0; i < 20; i++)
            {
                NestedProcess(i);
            }
            Log.ProcessEnd(processName: "SingleThread", processId: pId, metricName: "time", metricValue: DateTime.Now.Subtract(start).TotalSeconds);
        }

        public void MetricsParallel()
        {
            long pId = Log.ProcessStart(processName: "MultiThread");
            DateTime start = DateTime.Now;
            for (int i = 0; i < 5; i++)
            {
                ThreadPool.QueueUserWorkItem(NestedProcess, i);
            }
            Log.ProcessEnd(processName: "MultiThread", processId: pId, metricName: "time", metricValue: DateTime.Now.Subtract(start).TotalSeconds);
        }

        private static void NestedProcess(object i)
        {
            DateTime start = DateTime.Now;
            long pId2 = Log.ProcessStart(processName: "Nested Process" + i);
            for (int j = 0; j < 5; j++)
            {
                Log.ProcessEnd(processName: "Process (" + i + ", " + j + ")", metricName: "i*j", metricValue: (int) i * j);
            }
            Log.ProcessEnd(processName: "Nested Process:" + i, processId: pId2, metricName: "time", metricValue: DateTime.Now.Subtract(start).TotalSeconds);
        }

        [TestMethod]
        public void Counters()
        {
            Initializer();
            long counter = Status.GetNextCounter();
            Log.Info("Counter: " + counter);
            counter = Status.GetNextCounter();
            Log.Info("Counter: " + counter);
            for (int i = 0; i < 5; i++)
            {
                ThreadPool.QueueUserWorkItem(NestedCounter, i);
            }
        }

        private static void NestedCounter(object i)
        {
            long counter = Status.GetNextCounter();
            Log.Info("Parallel Counter" + i + ": " + counter);
        }

        [TestMethod]
        public void UserIdentifiers()
        {
            Initializer();
            var securitySRV = new SecuritySRV();
            var user= securitySRV.SelectByName("JackShephard");
            if ( user == null )
            {
                user = new User
                 {
                     UserName = "JackShephard",
                     Email = "JackShephard@island.com",
                     LastActivityDate = DateTime.Now,
                     LastLoginDate = DateTime.Now,
                     CreationDate = DateTime.Now,
                     LastLockoutDate = DateTime.Now,
                     LastPasswordChangedDate = DateTime.Now,
                     Password = "losties"
                 };
                securitySRV.InsertUser(user);
            }
            var identifier = securitySRV.CreateIdentifier(user.Id, DateTime.Now.AddDays(7));
            var userIdentifier = securitySRV.SelectIdentifierByGUID(identifier);
            if ( userIdentifier == null )
            {
                Assert.Fail("Identifier not found");
            }
            if ( userIdentifier.UsedDate.HasValue )
            {
                Assert.Fail("Alredy used");
            }
            if ( userIdentifier.ExpiryDate < DateTime.Now )
            {
                Assert.Fail("ExpiryDate in the past");
            }
            securitySRV.ConsumeUserIdentifier(userIdentifier.Id);
            var userIdentifier2 = securitySRV.SelectIdentifierByGUID(identifier);
            Assert.IsTrue(userIdentifier2.UsedDate.HasValue);

        }

        #region Nested type: Test
        public class Test
        {
            public string Message { get; set; }
            public string[] Array { get; set; }
            public List<string> List { get; set; }
        }
        #endregion

        [TestMethod]
        public void Email()
        {
            Less.Tools.Functions.SendEmailMessage("asunto", "cuerpo", "albertobasalo@hotmail.com", "Alberto GMail");
        }

    }
}